<?php declare(strict_types=1);

namespace MyPrayerJournal;

use BitBadger\PDODocument\DocumentException;
use BitBadger\PDODocument\DocumentList;
use DateTimeImmutable;
use DateTimeZone;

/**
 * User interface building blocks
 */
class UI
{
    /**
     * Generate a material icon
     *
     * @param string $name The name of the material icon
     * @return string The material icon wrapped in a `span` tag
     */
    public static function icon(string $name): string {
        return "<span class=material-icons>$name</span>";
    }

    /**
     * Render the journal items for the current user
     *
     * @throws DocumentException If any is encountered
     */
    public static function journal(): void
    {
        Layout::bareHead();
        $reqs = Request::forJournal();
        if ($reqs->hasItems()) { ?>
            <section id=journalItems class="row row-cols-1 row-cols-md-2 row-cols-lg-3 row-cols-xl-4 g-3" hx-target=this
                     hx-swap=outerHTML aria-label="Prayer Requests"><?php
            $spacer = '<span>&nbsp;</span>';
            foreach ($reqs->items() as /** @var Request $req */ $req) { ?>
                <div class=col>
                <div class="card h-100">
                    <div class="card-header p-0 d-flex" role=toolbar><?php
                        self::pageLink("/request/edit?id=$req->id", self::icon('edit'),
                            ['class' => 'btn btn-secondary', 'title' => 'Edit Request']); ?>
                        <?=$spacer?>
                        <button type=button class="btn btn-secondary" title="Add Notes" data-bs-toggle=modal
                                data-bs-target=#notesModal hx-get="/components/request/add-notes?id=<?=$req->id?>"
                                hx-target=#notesBody hx-swap=innerHTML><?=self::icon('comment');?></button>
                        <?=$spacer?>
                        <button type=button class="btn btn-secondary" title="Snooze Request" data-bs-toggle=modal
                                data-bs-target=#snoozeModal hx-get="/components/request/snooze?id=<?=$req->id?>"
                                hx-target=#snoozeBody hx-swap=innerHTML><?=self::icon('schedule');?></button>
                        <div class=flex-grow-1></div>
                        <button type=button class="btn btn-success w-25" hx-patch="/request/prayed?id=<?=$req->id?>"
                                title="Mark as Prayed"><?=self::icon('done');?></button>
                    </div>
                    <div class=card-body>
                        <p class=request-text><?=htmlentities($req->currentText());?>
                    </div>
                    <div class="card-footer text-end text-muted px-1 py-0">
                        <em><?php
                            $lastPrayed = $req->lastPrayed();
                            echo 'last ' . (is_null($lastPrayed) ? 'activity': 'prayed') . ' ';
                            self::relativeDate($lastPrayed ?? $req->history[0]->asOf); ?>
                        </em>
                    </div>
                </div>
                </div><?php
            } ?>
            </section><?php
        } else {
            UI::noResults('No Active Requests', '/request/edit?id=new', 'Add a Request', <<<'TEXT'
                You have no requests to be shown; see the &ldquo;Active&rdquo; link above for snoozed or deferred
                requests, and the &ldquo;Answered&rdquo; link for answered requests
                TEXT);
        }
        Layout::bareFoot();
    }

    /**
     * Create a card when there are no results found
     */
    public static function noResults(string $heading, string $link, string $buttonText, string $text): void
    { ?>
        <div class=card>
        <h5 class=card-header><?=$heading?></h5>
        <div class="card-body text-center">
            <p class=card-text><?=$text?></p><?php
            self::pageLink($link, $buttonText, ['class' => 'btn btn-primary']); ?>
        </div>
        </div><?php
    }

    /**
     * Generate a link to a page within myPrayerJournal
     *
     * @param string $href The URL for the link
     * @param string $text The text for the link
     * @param array $attrs Any additional attributes that should be placed on the `a` tag
     */
    public static function pageLink(string $href, string $text, array $attrs = []): void
    { ?>
        <a href="<?=$href?>" hx-get="<?=$href?>" hx-target=#top hx-swap=innerHTML hx-push-url=true<?php
        foreach ($attrs as $key => $value) echo " $key=\"" . htmlspecialchars($value) . "\""; ?>><?=$text?></a><?php
    }

    // Thanks, William Entriken!  https://stackoverflow.com/a/42446994/276707
    /** @var array Periods for the relative date/time display */
    private static array $periods = [
        [           60,           1, '%s seconds ago', 'a second ago'],
        [       60*100,          60, '%s minutes ago', 'one minute ago'],
        [      3600*70,        3600, '%s hours ago',   'an hour ago'],
        [   3600*24*10,     3600*24, '%s days ago',    'yesterday'],
        [   3600*24*30,   3600*24*7, '%s weeks ago',   'a week ago'],
        [3600*24*30*30,  3600*24*30, '%s months ago',  'last month'],
        [          INF, 3600*24*265, '%s years ago',   'last year']
    ];

    /**
     * @throws \Exception
     */
    public static function relativeDate(string $date): void
    {
        $parsed = new DateTimeImmutable($date);
        $diff   = time() - $parsed->getTimestamp();
        foreach (self::$periods as $period) {
            if ($diff > $period[0]) continue;
            $diff = floor($diff / $period[1]);
            $value = $diff > 1 ? sprintf($period[2], $diff) : $period[3];
            $inZone = $parsed->setTimezone(new DateTimeZone($_SERVER['HTTP_X_TIME_ZONE'] ?? 'Etc/UTC'));
            echo '<span title="' . date_format($inZone, 'l, F j, Y \a\t g:ia T') . '">' . $value . '</span>';
            break;
        }
    }

    public static function requestList(DocumentList $reqs): void
    {
        $btnClass = "btn btn-light mx-2";
        /// Create a request within the list
     /*   let reqListItem now tz req =
        let isFuture instant = defaultArg (instant |> Option.map (fun it -> it > now)) false
    let reqId      = RequestId.toString req.RequestId
    let isSnoozed  = isFuture req.SnoozedUntil
    let isPending  = (not isSnoozed) && isFuture req.ShowAfter
    let restoreBtn (link : string) title =
        button [ btnClass; _hxPatch $"/request/{reqId}/{link}"; _title title ] [ icon "restore" ] */ ?>
        <div class=list-group><?php
            foreach ($reqs->items() as /** @var Request $req */ $req) { ?>
                <div class="list-group-item px-0 d-flex flex-row align-items-start" hx-target=this
                     hx-swap=outerHTML><?php
                    self::pageLink("/request/full?id=$req->id", self::icon('description'),
                        ['class' => $btnClass, 'title' => 'View Full Request']);
                    if (!$req->isAnswered()) {
                        self::pageLink("/request/edit?id=$req->id", self::icon('edit'),
                            ['class' => $btnClass, 'title' => 'Edit Request']);
                    }
//        if   isSnoozed then restoreBtn "cancel-snooze" "Cancel Snooze"
//        elif isPending then restoreBtn "show"          "Show Now"

                    echo '<p class="request-text mb-0">' . $req->currentText();
//            if isSnoozed || isPending || isAnswered then
//                br []
//                small [ _class "text-muted" ] [
//                    if   isSnoozed then   [ str "Snooze expires ";       relativeDate req.SnoozedUntil.Value now tz ]
//                    elif isPending then   [ str "Request appears next "; relativeDate req.ShowAfter.Value    now tz ]
//                    else (* isAnswered *) [ str "Answered ";             relativeDate req.AsOf               now tz ]
//                    |> em []
//                ]
?>
                </div><?php
            } ?>
        </div><?php
    }
}
